;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                 LCD-Routinen                ;;
;;                 ============                ;;
;;              (c)andreas-s@web.de            ;;
;;                                             ;;
;; 4bit-Interface                              ;;
;; DB4-DB7:       PC3-PC0                      ;;
;; RS:            PB0                          ;;
;; E:             PB2                          ;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 

mirrordata:
  mov r1,temp1  
  bst r1, 7
  bld r2, 0
  bst r1, 6
  bld r2, 1
  bst r1, 5
  bld r2, 2
  bst r1, 4
  bld r2, 3
  bst r1, 3
  bld r2, 4
  bst r1, 2
  bld r2, 5
  bst r1, 1
  bld r2, 6
  bst r1, 0
  bld r2, 7
  mov temp1,r2
ret

 ;sendet ein Datenbyte an das LCD
lcd_data:
           rcall mirrordata
		   
		   
		   
		   mov temp2, temp1             ; "Sicherungskopie" fr
                                        ; die bertragung des 2.Nibbles
           swap temp1                   ; Vertauschen
           andi temp1, 0b00001111       ; oberes Nibble auf Null setzen
           sbi PORTB,2
           out PORTC, temp1             ; ausgeben
           rcall lcd_enable             ; Enable-Routine aufrufen
                                        ; 2. Nibble, kein swap da es schon
                                        ; an der richtigen stelle ist
           andi temp2, 0b00001111       ; obere Hlfte auf Null setzen 
           sbi PORTB,2                  ; entspricht 0b00010000
           out PORTC, temp2             ; ausgeben
           rcall lcd_enable             ; Enable-Routine aufrufen
           rcall delay50us              ; Delay-Routine aufrufen
           ret 
		   
		   
		   ; zurck zum Hauptprogramm
 
 ; sendet einen Befehl an das LCD
lcd_command:                            ; wie lcd_data, nur ohne RS zu setzen
           rcall mirrordata
		   cbi PORTB,2
		   mov temp2, temp1
           swap temp1
           andi temp1, 0b00001111
           out PORTC, temp1
           rcall lcd_enable
           andi temp2, 0b00001111
           out PORTC, temp2
           rcall lcd_enable
           rcall delay50us
           ret
 
 ; erzeugt den Enable-Puls
lcd_enable:
           sbi PORTB, 2                 ; Enable high
           nop                          ; 3 Taktzyklen warten
           nop
           nop
	       nop                          ; 3 Taktzyklen warten
           nop
           nop	    	
		   cbi PORTB, 2                 ; Enable wieder low
           ret                          ; Und wieder zurck                     
 
 ; Pause nach jeder bertragung
delay50us:                              ; 50us Pause
           ldi  temp2, $03
dWGLOOP0:   ldi  temp3, $20
dWGLOOP1:   dec  temp3
           brne dWGLOOP1
           dec  temp2
           brne dWGLOOP0
           ret                       ; wieder zurck
 
 ; Lngere Pause fr manche Befehle
delay5ms:                               ; 5ms Pause
           ldi  temp2, $01
WGLOOP0:   ldi  temp3, $64
WGLOOP1:   dec  temp3
           brne WGLOOP1
           dec  temp2
           brne WGLOOP0
           ret                          ; wieder zurck
 
 ; Initialisierung: muss ganz am Anfang des Programms aufgerufen werden




lcd_init:
           ldi  temp3,5
powerupwait:
           rcall  delay5ms
           dec  temp3
           brne powerupwait
           ldi temp1, 0b00110000        ; muss 3mal hintereinander gesendet
           
		   out PORTC, temp1             ; werden zur Initialisierung
           rcall lcd_enable             ; 1
           rcall delay5ms 
       	   rcall lcd_enable             ; 2
           rcall delay50us
           rcall delay50us
           rcall delay50us
           rcall lcd_enable             ; und 3!
           rcall delay5ms

           ldi temp1, 0b00110000        ; 4Bit / 2 Zeilen / 5x8
          
		   out PORTC, temp1
           rcall lcd_enable
           rcall delay50us
           
		   ldi temp1, 0b10000000        ; ddram auf anfang
           out PORTC, temp1
           rcall lcd_enable
           rcall delay50us
           


           ldi temp1, 0b00001111        ; Display on 
           out PORTC, temp1
           rcall lcd_enable
           rcall delay50us
       
           ldi temp1, 0b00000001        ; Display Clear
           out PORTC, temp1
           rcall lcd_enable
           rcall delay50us
        
           ldi temp1, 0b00000110        ; entry mode, no shift, increase
           out PORTC, temp1
           rcall lcd_enable
           rcall delay50us
      
               
          
           ret




 ; Sendet den Befehl zur Lschung des Displays
lcd_clear:
           ldi temp1, 0b00000001   ; Display lschen
           cbi portb,0
		   cbi PORTB,2
		   rcall lcd_command
           rcall delay5ms
           ret
 
 ; Sendet den Befehl: Cursor Home
lcd_home:
           ldi temp1, 0b00000010   ; Cursor Home
           rcall lcd_command
           rcall delay5ms
           ret
lcd_reset:
         rcall        delay5ms
         rcall        delay5ms
         rcall        delay5ms

         cbi          portb,0       ; Register auf Control

         ldi          temp1,$30
         out          portc,temp1

         rcall        lcd_enable
         rcall        delay5ms

         rcall        lcd_enable
         rcall        delay5ms

         rcall        lcd_enable
         rcall        delay5ms

         ret
